Class: Module

Inherits:
Object show all
Defined in:
lib/empirical/class_callbacks_processor.rb

Overview

Ensure the new callback methods exist on the base classes. Developers can define their own callback methods in their classes/modules.

Instance Method Summary collapse

Instance Method Details

#module_definedObject



6
7
# File 'lib/empirical/class_callbacks_processor.rb', line 6

def module_defined
end