Class: Empirical::VoidClass
- Inherits:
- BasicObject
- Defined in:
- lib/empirical.rb
Instance Method Summary collapse
Dynamic Method Handling
This class handles dynamic methods through the method_missing method
#method_missing(method_name) ⇒ Object
20 21 22 |
# File 'lib/empirical.rb', line 20 def method_missing(method_name, ...) ::Kernel.raise "The method `#{method_name}` was called on void. Methods that explicitly declare a void return type should not have their return values used for anything." end |